mod Mterm logic: 'cl'

incl Mtex

incl Mtexgr

incl Maux

rem 
  \para \bf  Arithmetization of recursive terms.  \end 

fun Vt 'Tex_f1_gr_vt'
  Vt(i) = 0,i

fun/0 Zt 'Tex_f0_gr_zt'
  Zt = 1,0

fun St 'Tex_f1_gr_st'
  St(t) = 2,t

fun Pt 'Tex_f1_gr_pt'
  Pt(t) = 3,t

fun/3 Dt 'Tex_f3_gr_dt'
  Dt(t1,t2,t3) = 4,t1,t2,t3

fun/2 Act 'Tex_f2_gr_act'
  Act(t1,t2) = 5,t1,t2

fun/2 Apt 'Tex_f2_gr_apt'
  Apt(e,ts) = 6,e,ts

rem 
  \para \bf  Arithmetization of recursive function symbols.  \end 

fun Fs 'Tex_f2_gr_fs'
  Fs(n) = 0,n

fun/2 Ls 'Tex_f2_gr_ls'
  Ls(n,t) = 1,n,t

rem 
  \para \bf  Codes of numerals.  \end 

pred Nm 
  Nm(Zt)
  Nm St(t) <- Nm(t)

pred Nms 
  Nms(0)
  Nms(t,ts) <- Nm(t) & Nms(ts)

rem 
  \para \bf  Contraction function.  \end 

pred Is_apt 
  Is_apt Apt(e,ts)

rem 
  \para Note that the auxiliary predicate \ft Is_apt(t) \end has the following 
  basic property: 
  \eq* 
    Is_apt(t) <-> \e n\e ts t = Apt(e,ts)
  \end

fun/2 Cntr 'Tex_f2_gr_cntr'
  Cntr(t1,t2) = Apt(e,ts++(t2,0)) <- Is_apt(t1) & Nm(t2) & t1 = Apt(e,ts)
  Cntr(t1,t2) = Act(t1,t2) <- Is_apt(t1) & ~Nm(t2)
  Cntr(t1,t2) = Act(t1,t2) <- ~Is_apt(t1)

rem 
  \para \bf  Recursive terms.  \end \header* pred/2 Tm1  \end The binary 
  predicate \ft Tm1(t,n) \end holds if \ft t \end is the code of some r. term 
  \ft Tex0_tau \end which is in the variables 
  \ft x1,Texm_f0_dotsc,Texm_f2_sub(x,n) \end and in the \ft n \end-ary recursor. 

pred/3 Tm2 
  Tm2(Vt(i),n,0) <- 1 <= i & i <= n
  Tm2(Zt,n,0)
  Tm2(St(t),n,0) <- Tm2(t,n,0)
  Tm2(Pt(t),n,0) <- Tm2(t,n,0)
  Tm2(Dt(t1,t2,t3),n,0) <- Tm2(t1,n,0) & Tm2(t2,n,0) & Tm2(t3,n,0)
  Tm2(Act(t1,t2),n,k) <- Tm2(t2,n,0) & Is_apt(t1) & ~Nm(t2) & Tm2(t1,n,k+1)
  Tm2(Act(t1,t2),n,k) <- Tm2(t2,n,0) & ~Is_apt(t1) & Tm2(t1,n,k+1)
  Tm2(Apt(Fs(n1),ts),n,k) <- n1 = n & n >= 1 & Nms(ts) & L(ts)+k = n
  Tm2(Apt(Ls(m,t),ts),n,k) <- m >= 1 & Nms(ts) & L(ts)+k = m & Tm2(t,m,0)

pred/2 Tm1 
  Tm1(t,n) <-> Tm2(t,n,0)

rem 
  \para \bf  Closed recursive terms.  \end The predicate \ft Ctm(t) \end holds 
  if \ft t \end is the code of some closed r. term. 

pred/2 Ctm1 
  Ctm1(Zt,0)
  Ctm1(St(t),0) <- Ctm1(t,0)
  Ctm1(Pt(t),0) <- Ctm1(t,0)
  Ctm1(Dt(t1,t2,t3),0) <- Ctm1(t1,0) & Ctm1(t2,0) & Ctm1(t3,0)
  Ctm1(Act(t1,t2),k) <- Ctm1(t2,0) & Is_apt(t1) & ~Nm(t2) & Ctm1(t1,k+1)
  Ctm1(Act(t1,t2),k) <- Ctm1(t2,0) & ~Is_apt(t1) & Ctm1(t1,k+1)
  Ctm1(Apt(Ls(m,t),ts),k) <- m >= 1 & Nms(ts) & L(ts)+k = m & Tm1(t,m)

pred Ctm 
  Ctm(t) <-> Ctm1(t,0)

rem 
  \para \bf  Well-formed recursive indices.  \end The predicate \ft Cdf(e) \end 
  holds if the number \ft e \end is a well-formed recursive index, ie it is the 
  code of some defined recursive function symbol. 

pred Cdf 
  Cdf Ls(n,t) <- n >= 1 & Tm1(t,n)

rem 
  \para \bf  Size of recursive terms.  \end 

fun Sznm 
  Sznm(Zt) = 1
  Sznm St(t) = Sznm(t)+1

fun Sznms 
  Sznms(0) = 0
  Sznms(t,ts) = Sznm(t)+Sznms(ts)

fun Sz 
  Sz Vt(i) = 1
  Sz(Zt) = 1
  Sz St(t) = Sz(t)+1
  Sz Pt(t) = Sz(t)+1
  Sz Dt(t1,t2,t3) = Sz(t1)+Sz(t2)+Sz(t3)+1
  Sz Act(t1,t2) = Sz(t1)+Sz(t2)
  Sz Apt(Fs(n),ts) = Sznms(ts)+1
  Sz Apt(Ls(n,t),ts) = Sznms(ts)+1

rem 
  \para \bf  Formats.  \end The predicate \ft Ftm \end is the union format and 
  it can be used to control the display of the codes of r. terms and of r. 
  function symbols in queries. 

pred Ff2 
  Ff2 Fs(n) <- N(n)
  Ff2 Ls(n,t) <- N(n) & M(t)

pred Ftm2 
  Ftm2 Vt(i) <- N(i)
  Ftm2(Zt)
  Ftm2 St(t) <- Ftm2(t)
  Ftm2 Pt(t) <- Ftm2(t)
  Ftm2 Dt(t1,t2,t3) <- Ftm2(t1) & Ftm2(t2) & Ftm2(t3)
  Ftm2 Act(t1,t2) <- Ftm2(t1) & Ftm2(t2)
  Ftm2 Apt(e,t) <- Ff2(e) & Nms(t)

pred Ff1 
  Ff1 Fs(n) <- N(n)
  Ff1 Ls(n,t) <- N(n) & Ftm2(t)

pred Ftm1 
  Ftm1 Vt(i) <- N(i)
  Ftm1(Zt)
  Ftm1 St(t) <- Ftm1(t)
  Ftm1 Pt(t) <- Ftm1(t)
  Ftm1 Dt(t1,t2,t3) <- Ftm1(t1) & Ftm1(t2) & Ftm1(t3)
  Ftm1 Act(t1,t2) <- Ftm1(t1) & Ftm1(t2)
  Ftm1 Apt(e,t) <- Ff1(e) & Nms(t)

pred Ff 
  Ff Fs(n) <- N(n)
  Ff Ls(n,t) <- N(n) & Ftm1(t)

pred Ftm 
  Ftm Vt(i) <- N(i)
  Ftm(Zt)
  Ftm St(t) <- Ftm(t)
  Ftm Pt(t) <- Ftm(t)
  Ftm Dt(t1,t2,t3) <- Ftm(t1) & Ftm(t2) & Ftm(t3)
  Ftm Act(t1,t2) <- Ftm(t1) & Ftm(t2)
  Ftm Apt(e,t) <- Ff(e) & Nms(t)

